xquery version "3.1";

declare namespace repo="http://exist-db.org/xquery/repo";
declare namespace xconf="http://exist-db.org/collection-config/1.0";
declare namespace sm="http://exist-db.org/xquery/securitymanager";



(: The following external variables are set by the repo:deploy function :)
(: the target collection into which the app is deployed :)
declare variable $target external;
declare variable $name := tokenize($target, "/")[last()];
declare variable $app-path := $target || "/";
declare variable $data-path := "/db/data/" || $name || "/";
declare variable $credentials-file := "file:///var/lib/textgrid/bdn-credentials.xml";
declare variable $tg-login := "file:///var/lib/textgrid/bdn-tg-login.xml";

(: TODO: add /log and /resources collections to data:)
(:
 : ERROR: exerr:ERROR Could not locate collection: /db/data/sade/logs/ [at line 382, column 10] In function: bdn-utils:create-log(xs:string) [123:9:/db/apps/sade/modules/bdn/utils.xqm] xml-utils:main-single-file(xs:string) [308:5:/db/apps/sade/modules/bdn/xml-utils.xqm] tgconnect:prerender(xs:string) [142:35:/db/apps/sade/modules/textgrid/connect.xqm] tgconnect:publish(xs:string, xs:string, xs:string, xs:string, xs:string, xs:string, xs:string, xs:boolean) [22:23:/db/apps/sade/modules/textgrid/connect.xqm]
 :)

declare variable $tg-uris :=
  <tg-objects>
    <tg-project id="TGPR-60c6a362-f1f6-1c81-544b-53837e6c4fb1">
      <tg-uri>3qjw2.2</tg-uri>
      <tg-uri>3qrcn.0</tg-uri>
    </tg-project>
  </tg-objects>
;



(:~
 : Helper function to get all function names declared in the application.
 : Uses externally declared variable $target.
 :
 : @author Mathias Göbel
 : @since 3.1.0
 :)
declare function local:get-function-names()
as xs:string* {
for $modulePath in collection($app-path)/base-uri()[ends-with(., ".xqm")]
let $functions := inspect:module-functions( xs:anyURI($modulePath) ) (: QNames :)
return
  for $function in $functions
  let $functionQName := function-name($function)
  return
      prefix-from-QName($functionQName) || ":" || local-name-from-QName($functionQName)
};

(:~
 : Formats the test coverage output - mainly used by GitLab test coverage parser.
 :
 : @author Mathias Göbel
 : @since 3.1.0
 : @see https://docs.gitlab.com/ee/user/project/pipelines/settings.html#test-coverage-parsing
 :   :)
declare function local:format-test-rate-output($value as xs:decimal)
as xs:string {
    "coverage: " || ($value * 100) => format-number("###.00") || "%"
};

(:~
 : Returns the relative amount of tests for all function in this package
 : @param $tests – a set of unit test results in the JUnit XML format
 :
 : @author Mathias Göbel
 : @since 3.1.0
 : @see https://www.ibm.com/support/knowledgecenter/en/SSQ2R2_14.1.0/com.ibm.rsar.analysis.codereview.cobol.doc/topics/cac_useresults_junit.html
 :  :)
declare function local:test-function-rate($tests as element(testsuites)*)
as xs:string {
    (: is function cardinality an issue? thats why disstinct-value is used :)
    let $functionNames := (local:get-function-names()) => distinct-values() => count()
    let $testedFunctionNames := ($tests//testcase/string(@class)) => distinct-values() => count()
    let $coverage :=
        if($testedFunctionNames = 0) then 0 else $testedFunctionNames div $functionNames
    return
        local:format-test-rate-output($coverage)
};

(:~
 : Returns the relative amount of successfull tests
 : @param $tests – a set of unit test results in the JUnit XML format
 :
 : @author Mathias Göbel
 : @since 3.1.0
 : @see https://www.ibm.com/support/knowledgecenter/en/SSQ2R2_14.1.0/com.ibm.rsar.analysis.codereview.cobol.doc/topics/cac_useresults_junit.html
 :  :)
declare function local:test-success-rate($tests as element(testsuites)*)
as xs:string {
  let $allFailures := $tests//testsuite/@failures => sum()
  let $failsTriggered := $tests//testcase[@name=("FAIL", "fail")] => count()
  let $failures := $allFailures - $failsTriggered
  let $errors := $tests//testsuite/@errors => sum()
  let $pending := $tests//testsuite/@pending => sum()

  let $testsDone := $tests//testsuite/@tests => sum()

  let $testCoverage := ($testsDone - sum( ($failures, $errors, $pending) )) div $testsDone
  return
    local:format-test-rate-output($coverage)
};

(:~
 : Checks for a credentials file, creates the the given user accounts and
 : disables user 'admin'.
 : @param $credentials-file – a file containing credentials for user accounts
 : @return ?
 :
 : @author Stefan Hynek
 : @since 4.3.0
 :
 : TODO: needs to be moved to a step after "<finish>" because of
 : exerr:ERROR org.exist.EXistException: Database instance 'exist' is not available
 : maybe this is not possible during autodeployment
 :)
(:declare function local:credentials($credentials-file)
{
  if ( doc-available($credentials-file) and exists(doc($credentials-file)//username) )
  then
  (
    let $credentials := doc($credentials-file)
    let $secure :=
      if ( sm:is-account-enabled("admin") )
      then
      (
        sm:set-account-enabled("admin", false()),
        util:log-app("info", "bdn-sade.log", "Disabled user 'admin'.")
      )
      else ( util:log-app("info", "bdn-sade.log", "User 'admin' already disabled.") )
      for $account in $credentials//account
      return
        if ( sm:user-exists($account/username) )
        then ( util:log-app("info", "bdn-sade.log", "User '" || $account/username || "' already exists.") )
        else
        (
          sm:create-account
          (
            $account/username,
            $account/password,
            $account/groups,
            $account/fullname,
            $account/description
          ),
          util:log-app("info", "bdn-sade.log", "Created user '" || $account/username || "'.")
        )
  )
  else ( util:log-app("info", "bdn-sade.log", $credentials-file || " does not exist. Enabled user 'admin' with empty password.") )
};
:)

let $project-name := tokenize($app-path, "/")[last()]
let $log := util:log("info", "installing " || $project-name)

let $fileSeparator := util:system-property("file.separator")
let $system-path := system:get-exist-home() || $fileSeparator

(: third parameter preserves the attributes from the source collection that is deployed with the permissions set in repo.xml :)
let $create-data-collection := xmldb:copy-collection($app-path || "textgrid/", $data-path, true())


let $use-template :=
(: TODO move to /data:)
    if(doc-available($data-path || "textgrid/data/collection-template.xconf"))
    then xmldb:rename($data-path || "textgrid/data", "collection-template.xconf", "collection.xconf")
    else true()
let $data-xconf := $data-path || "textgrid/data/collection.xconf"

(: prepare empty Lucene Config files :)
let $prepare-config := (
      util:eval("file:serialize-binary( xs:base64Binary(util:base64-encode(""# default config file&#10;ä,ae&#10;ö,oe&#10;ü,ue"")), $system-path || $project-name || '-charmap.txt')"),
      util:eval("file:serialize-binary( xs:base64Binary(util:base64-encode(""# default config file&#10;Backfisch, Tochter, Mädchen"")), $system-path || $project-name || '-synonyms.txt')")
      )
let $print-message :=
    if($project-name != "sade") then true() else
        util:log("info", "&#27;[48;2;"|| "255;0;0" ||"m&#27;[38;2;0;0;0m "
        || "The search module was set up correctly. A restart of the database is "
        || "required to enable the charmap and synonym table." || " &#27;[0m" )


(: rewrite the local path in index config  :)
let $rewrite-mappings := update value
                doc( $data-xconf )
                  //xconf:lucene/xconf:analyzer/xconf:param[@name="mappings"]
                  /@value
                with $system-path || $project-name || "-charmap.txt"
let $rewrite-synonyms := update value
                doc( $data-xconf )
                  //xconf:lucene/xconf:analyzer/xconf:param[@name="synonyms"]
                  /@value
                with $system-path || $project-name || "-synonyms.txt"
(: TODO move to /data:)
let $store-xconf := xmldb:store("/db/system/config" || $data-path || "/textgrid/data", "collection.xconf", doc( $data-xconf ))
let $reindex := xmldb:reindex($data-path || "textgrid/data")

(: we have to test for writable path :)
let $path := $system-path
let $user := util:system-property("user.name")
let $message1 := $path || " is not available. Create it and make sure "
                || $user || " can write there."
let $message2 := "Could not write to " || $path || "."

(:let $secure := local:credentials($credentials-file):)

return ()
